// Generated by CoffeeScript 1.10.0

/*
description: test jssgf.js
author: ICHIKAWA, Yuji
copyright: 2014 (C) ICHIKAWA, Yuji (New 3 Rs)
 */
var assert, parser;

assert = require('assert');

parser = require('./jssgf').parser;

describe('parser', function() {
  describe('collection', function() {
    it('should return the smallest collection', function() {
      return assert.deepEqual(parser.parse('(;)'), [
        {
          _children: []
        }
      ]);
    });
    it('should includes plural game tree', function() {
      return assert.deepEqual(parser.parse('(;)(;)'), [
        {
          _children: []
        }, {
          _children: []
        }
      ]);
    });
    return it('should includes plural game tree', function() {
      return assert.deepEqual(parser.parse('(;)\n(;)'), [
        {
          _children: []
        }, {
          _children: []
        }
      ]);
    });
  });
  describe('game tree', function() {
    it('should includes empty value of property ', function() {
      return assert.deepEqual(parser.parse('(;TB)'), [
        {
          _children: [],
          TB: null
        }
      ]);
    });
    it('should includes empty value of property ', function() {
      return assert.deepEqual(parser.parse('(;TB FF[4])'), [
        {
          _children: [],
          TB: null,
          FF: '4'
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.parse('(;FF[4])'), [
        {
          _children: [],
          FF: '4'
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.parse('(;FF[4]GM[1])'), [
        {
          _children: [],
          FF: '4',
          GM: '1'
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.parse('(;FF[4]GM[1]EV[])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: ''
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.parse('(;FF[4]GM[1]EV[ ])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: ' '
        }
      ]);
    });
    it('should treats a soft line break', function() {
      return assert.deepEqual(parser.parse('(;FF[4]GM[1]EV[a\\\nb])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: 'ab'
        }
      ]);
    });
    it('should treats an escaped ]', function() {
      return assert.deepEqual(parser.parse('(;FF[4]GM[1]EV[\\]])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: ']'
        }
      ]);
    });
    it('should treats an escaped ]', function() {
      return assert.deepEqual(parser.parse('(;FF[4]GM[1]EV[()])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: '()'
        }
      ]);
    });
    return it('should  an escaped ]', function() {
      return assert.deepEqual(parser.parse('(;DT[2015-09-03];B[pd](;W[nq])(;W[aa]))'), [
        {
          _children: [
            {
              _children: [
                {
                  _children: [],
                  W: 'nq'
                }, {
                  _children: [],
                  W: 'aa'
                }
              ],
              B: 'pd'
            }
          ],
          DT: '2015-09-03'
        }
      ]);
    });
  });
  describe('stringify', function() {
    it('should return string', function() {
      var sgf;
      sgf = '(;FF[4])';
      return assert.equal(parser.stringify(parser.parse(sgf)), sgf);
    });
    return it('should make FF prior', function() {
      var sgf;
      sgf = '(;GM[1]FF[4])';
      return assert.equal(parser.stringify(parser.parse(sgf)), '(;FF[4]GM[1])');
    });
  });
  describe('isSgf', function() {
    return it('should return true', function() {
      var sgf;
      sgf = '(;FF[4])';
      return assert.equal(parser.isSgf(sgf), true);
    });
  });
  return describe('nthMoveNode', function() {
    it('should return root', function() {
      var root;
      root = parser.parse('(;FF[4]SZ[19];B[aa])')[0];
      return assert.equal(parser.nthMoveNode(root, 0), root);
    });
    it('should return first move node', function() {
      var root;
      root = parser.parse('(;FF[4]SZ[19];B[aa])')[0];
      return assert.equal(parser.nthMoveNode(root, 1), root._children[0]);
    });
    return it('should return first move node', function() {
      var root;
      root = parser.parse('(;FF[4]SZ[19];B[aa];W[bb];B[cc];W[dd])')[0];
      return assert.equal(parser.nthMoveNode(root, Infinity).W, 'dd');
    });
  });
});
